/**
 Find a linear ranking function for the program

 typedef unsigned long ULONG;

int main(void)
{
  ULONG BiasedDirentOffset;

  do {

      ;

      BiasedDirentOffset >>= 4;

  } while (BiasedDirentOffset != 0);

  return 0;
}

 The critical path to be checked is described by the following
 predicate:

   BiasedDirentOffset != 0 && BiasedDirentOffset == BiasedDirentOffset$pre >> 4

 In order to express this as a Presburger formula, we decompose
 the variable BiasedDirentOffset as follows:

   BiasedDirentOffset = BiasedDirentOffset1 * 16 + BiasedDirentOffset2 &
   BiasedDirentOffset2 >= 0 & BiasedDirentOffset2 < 16

 We end up with the following formula for the path:

   BiasedDirentOffset >= 0 & BiasedDirentOffset < 4 * 1024 * 1024 * 1024 &
   BiasedDirentOffset' >= 0 & BiasedDirentOffset' < 4 * 1024 * 1024 * 1024 &
   BiasedDirentOffset != 0 &
   BiasedDirentOffset = BiasedDirentOffset1 * 16 + BiasedDirentOffset2 &
   BiasedDirentOffset2 >= 0 & BiasedDirentOffset2 < 16 &
   BiasedDirentOffset' = BiasedDirentOffset'1 * 16 + BiasedDirentOffset'2 &
   BiasedDirentOffset'2 >= 0 & BiasedDirentOffset'2 < 16 &
   BiasedDirentOffset' = BiasedDirentOffset1
   

 (in unsigned 32-bit arithmetic)
 */

\existentialConstants {
  // the lambda-witness vectors

  // lambda1
  int l1, l2, l3, l4, l5, l6, l7, l8, l9, l10, l11, l12, l13, l14;
  // lambda2
  int m1, m2, m3, m4, m5, m6, m7, m8, m9, m10, m11, m12, m13, m14;

  int rankingCoeff1, rankingCoeff2, rankingCoeff3, lowerBound;
}

\problem {
  
//   lambda1 Ai' = 0

  l3 * -1 + l4 * 1 + l9 * 1 + l10 * -1 + l13 * -1 + l14 * 1 = 0
&
  l9 * -16 + l10 * 16 = 0
&
  l9 * -1 + l10 * 1 + l11 * -1 + l12 * 1 = 0
&

//   (lambda1 - lambda2) Ai = 0

  (l1 - m1) * -1 + (l2 - m2) * 1 + (l5 - m5) * 1 + (l6 - m6) * -1 = 0
&  
  (l5 - m5) * -16 + (l6 - m6) * 16 + (l13 - m13) * 1 + (l14 - m14) * -1 = 0
&  
  (l5 - m5) * -1 + (l6 - m6) * 1 + (l7 - m7) * -1 + (l8 - m8) * 1 = 0
&  

//   lambda2 (Ai + Ai') = 0

  m1 * (-1 + 0) + m2 * (1 + 0) + m3 * (0 + -1) + m4 * (0 + 1) +
  m5 * (1 + 0) + m6 * (-1 + 0) + m9 * (0 + 1) + m10 * (0 + -1) +
  m13 * (0 + -1) + m14 * (0 + 1) = 0
&
  m5 * (-16 + 0) + m6 * (16 + 0) + m9 * (0 + -16) + m10 * (0 + 16) +
  m13 * (1 + 0) + m14 * (-1 + 0) = 0
&
  m5 * (-1 + 0) + m6 * (1 + 0) + m7 * (-1 + 0) + m8 * (1 + 0) +
  m9 * (0 + -1) + m10 * (0 + 1) + m11 * (0 + -1) + m12 * (0 + 1) = 0
&

//   lambda2 bi < 0

  m1 * -1 + m2 * (4 * 1024 * 1024 * 1024 - 1) +
  m3 * 0 + m4 * (4 * 1024 * 1024 * 1024 - 1) +
  m8 * 15 + m12 * 15 < 0
&

//   lambda2 A1'

  rankingCoeff1 = m3 * -1 + m4 * 1 + m9 * 1 + m10 * -1 + m13 * -1 + m14 * 1
&
  rankingCoeff2 = m9 * -16 + m10 * 16
&
  rankingCoeff3 = m9 * -1 + m10 * 1 + m11 * -1 + m12 * 1
&
  lowerBound = -(l1 * -1 + l2 * (4 * 1024 * 1024 * 1024 - 1) +
                 l3 * 0 + l4 * (4 * 1024 * 1024 * 1024 - 1) +
                 l8 * 15 + l12 * 15)
&

//   ls non-negative

  l1 >= 0 &
  l2 >= 0 &
  l3 >= 0 &
  l4 >= 0 &
  l5 >= 0 &
  l6 >= 0 &
  l7 >= 0 &
  l8 >= 0 &
  l9 >= 0 &
  l10 >= 0 &
  l11 >= 0 &
  l12 >= 0 &
  l13 >= 0 &
  l14 >= 0 &
  m1 >= 0 &
  m2 >= 0 &
  m3 >= 0 &
  m4 >= 0 &
  m5 >= 0 &
  m6 >= 0 &
  m7 >= 0 &
  m8 >= 0 &
  m9 >= 0 &
  m10 >= 0 &
  m11 >= 0 &
  m12 >= 0 &
  m13 >= 0 &
  m14 >= 0
}