/**
 * This file is part of Princess, a theorem prover for Presburger
 * arithmetic with uninterpreted predicates.
 * <http://www.philipp.ruemmer.org/princess.shtml>
 *
 * Copyright (C) 2009 Philipp Ruemmer <ph_r@gmx.net>
 *
 * Princess is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Princess is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Princess.  If not, see <http://www.gnu.org/licenses/>.
 */

package ap.parser

import ap.terfor.{ConstantTerm, TermOrder}
import ap.parameters.{PreprocessingSettings, Param}

/**
 * Preprocess an InputAbsy formula in order to make it more suitable for
 * proving
 */
object Preprocessing {
  
  class PreprocessingException(msg : String) extends Exception(msg)

  def apply(f : IFormula,
            signature : Signature,
            settings : PreprocessingSettings) : (IFormula, Signature) = {
    val f2 = EquivExpander(f)
    val functionEncoder = new FunctionEncoder
    val (f3, order3) = functionEncoder(f2, signature.order)
    val f4 = functionEncoder.axioms --> f3
    val f5 = Param.CLAUSIFIER(settings) match {
      case Param.ClausifierOptions.None => f4
      case Param.ClausifierOptions.Simple => SimpleClausifier(f4)
    }
    (f5, signature updateOrder order3)
  }
  
}
